[cylc]
   UTC mode = True
[scheduling]
    initial cycle point = 20140107
    final cycle point = 20140118
    runahead limit = P1D # enforce sequential running
    [[graph]]
        # daily cycling *with a non-zero hour* is a more stringent test
        20140110T06/P3D = "a"
[runtime]
    [[a]]
        script = true

[visualization]
    initial cycle point = 20140107
    final cycle point = 20140118
