[cylc]
    UTC mode = True
    [[events]]
        inactivity = PT10S
        abort on inactivity = True
        abort on timeout = True

[scheduling]
    [[graph]]
        R1 = foo

[runtime]
    [[foo]]
        script = """
trap '' EXIT
exit
"""
