[cylc]
    [[events]]
        startup handler = "false"
        abort if startup handler fails = True

[scheduling]
    [[graph]]
        R1 = "foo"

[runtime]
    [[root]]
        script = "true" # no sleep
