[meta]
    title = "Test validation of simple multiple inheritance"

    description = """Bug identified at 5.1.1-314-g4960684."""

[scheduling]
[[graph]]
R1 = """foo"""
[runtime]
[[FOO]]
[[BAR]]
[[foo]]
inherit = FOO, BAR
