[cylc]
   [[events]]
       abort on stalled = True
       abort on inactivity = True
       inactivity = PT3M
[scheduling]
    [[graph]]
        R1 = a-task
[runtime]
    [[a-task]]
        script = """
# Write to task stdout log
echo "the quick brown fox"
# Write to task stderr log
echo "jumped over the lazy dog" >&2
# Write to a custom log file
echo "drugs and money" > ${CYLC_TASK_LOG_ROOT}.custom-log
# Generate a warning message in the suite log.
cylc task message -p WARNING 'marmite and squashed bananas'
"""
