# THIS FILE IS PART OF THE CYLC SUITE ENGINE.
# Copyright (C) NIWA & British Crown (Met Office) & Contributors.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


def echo(*args, **kwargs):
    """An xtrigger function that prints its arguments and never succeeds.

    This may be a useful aid to understanding how xtriggers work. Try returning
    True (success) and some results dict to pass on to dependent tasks.

    Returns
        tuple: (False, {})

    """
    print("echo: ARGS:", args)
    print("echo: KWARGS:", kwargs)
    return False, {}
