[cylc]
    force run mode = dummy
    [[reference test]]
        expected task failures = a.1, b.1, c.1
[scheduling]
    [[graph]]
        R1 = """a:fail => b
            b:fail => c
            c:fail => !a & !b & !c"""
[runtime]
    [[a, b, c]]
        [[[simulation]]]
            fail cycle points = all
