[cylc]
    [[reference test]]
        expected task failures = foo.1
    [[events]]
        abort on stalled = True
[scheduling]
    [[graph]]
        R1 = foo
[runtime]
    [[foo]]
        err-script = echo "$1 ${FOO}"
        script = test "${CYLC_TASK_SUBMIT_NUMBER}" -ge 2
        [[[environment]]]
            FOO = foo bar baz qux
        [[[job]]]
            execution retry delays = PT0S
