[meta]
    title = "reference test suite: zig-zag intercycle dependencies"

    description = """
Task A should only run at 0, 12 hours; Task B at 6, 18"""

[cylc]
    UTC mode = True

[scheduling]
    initial cycle point = 20120101T00
    final cycle point = 20120101T18
    [[graph]]
        T00,T12 = "A"
        T06,T18 = "A[-PT6H] => B"
[runtime]
    [[root]]
        script = "true" # fast
