#!jinja2
[cylc]
   [[events]]
       abort on stalled = True
   [[reference test]]
       expected task failures = t1.1

[scheduling]
    [[graph]]
        R1=t1

[runtime]
    [[t1]]
        script=echo "${CYLC_HAS_NO_SUCH_VARIABLE}" | tee 'no-such-variable.txt'
