[meta]
    title = Test for the get-cylc-version command.
    description = """A task compares its own cylc version to that running the
    test suite (should be the same)."""

[scheduling]
    [[graph]]
            R1 = foo

[runtime]
    [[root]]
        script = """
diff -u <(cylc --version) <(cylc get-cylc-version "${CYLC_SUITE_NAME}")
"""
