[meta]
    title = Job script: suite to environment test

[scheduling]
    [[graph]]
        R1 = foo

[runtime]
    [[foo]]
        script = printenv FOO BAR BAZ
        [[[remote]]]
            host = localhost
