#!jinja2
[cylc]
    UTC mode=True
    cycle point format = %Y
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT3M
        startup handler = cylc release '%(suite)s'
[scheduling]
    initial cycle point = 2016
    final cycle point = 2020
    [[graph]]
        P1Y=t1 & t2
[runtime]
    [[t1, t2]]
        script = sleep $((RANDOM % 2 + 2))
        [[[job]]]
            execution time limit = PT15S
