[meta]
    title = A suite to test an aborted edit run has no subsequent effect.
[cylc]
    [[events]]
        abort on timeout = True
        timeout = PT20S
[scheduling]
    [[graph]]
        R1 = victim => perpetrator
[runtime]
    [[victim]]
        script = echo HELLO
    [[perpetrator]]
        script = """
# Edit run to break victim, but abort after the edit.
cylc trigger --edit $CYLC_SUITE_NAME victim.1 << __END__
n
__END__
# Add a new line (for nicer output).
echo

# Give the state summary a chance to update.
sleep 2

# Test that the NN symbolic link is correct.
test $(readlink ${CYLC_SUITE_RUN_DIR}/log/job/1/victim/NN) = '01'

my-suite-state-summary-test \
    < <(cylc client -n "${CYLC_SUITE_NAME}" 'get_suite_state_summary')

# Then trigger victim - the original unedited task should run.
cylc trigger $CYLC_SUITE_NAME victim.1
"""
