[cylc]
    [[reference test]]
        expected task failures = c.1

[scheduling]
    [[graph]]
        R1 = """a => b => c
                   c:fail => cleaner
                   cleaner => d"""
[runtime]
    [[a,b]]
        script = true
    [[c]]
        script = false
    [[d]]
    [[cleaner]]
        script = cylc remove "$CYLC_SUITE_NAME" 'c.1' 'd.1'
