[cylc]
    cycle point time zone = Z
[scheduling]
    initial cycle point = 20001231T0100
    final cycle point = 20010114
    [[graph]]
        P1D = "foo[-P1D] => foo => bar"
        P4D = "baz[-P4D] => baz => qux"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 20001231T0100
    final cycle point = 20010114
