[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20140808T00
    [[graph]]
        R1 = cold_foo => foo
        T00 = foo => bar => baz
[runtime]
    [[FOO]]
    [[foo,bar,baz]]
        inherit = None, FOO
