[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20130101T00
    final cycle point = 20130103T00
    [[graph]]
        R1 = "foo"
        R1/T18 = "bar => baz"
        R6//PT6H = "wibble"
        T00,T06,T12,T18 = "foo[^] & qux[-PT6H] => qux"
[runtime]
    [[root]]
        script = "true"
