[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 1000W011
    final cycle point = 1000W064
    [[graph]]
        P1W = "foo[-P1W] => foo => bar"
        P3W = "baz[-P3W] => baz => qux"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 1000W011
    final cycle point = 1000W064
