#!jinja2
[cylc]
    UTC mode = True
    [[events]]
        timeout = PT30S
        abort on timeout = True
[scheduling]
    initial cycle point = 20100101T00
    final cycle point = 20100105T00
    [[graph]]
        # foo will fail and cause a stuck bar task at T00.
        PT1H = "foo => bar"
        # If wibble gets into the pool, it will demand a +PT6H raise
        # of the 'runahead limit'.
        {{ FUTURE_TRIGGER_START_POINT }}/PT6H = """
            baz[+PT6H] => wibble
            baz
        """
[runtime]
    [[foo]]
        script = false
    [[bar,baz,wibble]]
        script = true

[visualization]
    number of cycle points = 10
