[scheduling]
    initial cycle point = 20160101T00Z
    final cycle point = 20160102T00Z
    [[graph]]
        R1 = start => one
        PT1H = add => foo => boo => baz
[runtime]
    [[root]]
        script = true
