[cylc]
    UTC mode = True
[scheduling]
    initial cycle point=2020
    final cycle point=2020
    [[graph]]
        P1Y = """
            t1
            t2
        """
[runtime]
    [[t1]]
        script=true
    [[t2]]
        script=true
        [[[remote]]]
            host=$CYLC_TEST_HOST
