[cylc]
    [[parameters]]
         t = 1..5
         u = right,left
[scheduling]
    [[graph]]
        R1 = foo<t,u>
[runtime]
    [[root]]
        script = true
        [[[environment]]]
            PARAM1 = 'something'
    [[foo<t,u>]]
        [[[environment]]]
            PARAM1 = $CYLC_TASK_PARAM_t
            PARAM2 = $CYLC_TASK_PARAM_u
