[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 2014-01-01
    final cycle point = 2014-01-04
    [[graph]]
        R1 = "cold_foo"
        R1/T00 = "cold_foo[^] => foo_midnight"
        R1/T06 = "cold_foo[^] => foo_dawn"
        T00 = "foo_midnight[-P1D] => foo_midnight"
        T06 = "foo_dawn[-P1D] => foo_dawn"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 2014-01-01
    final cycle point = 2014-01-04
