[meta]
    title = Test cylc hold/release remembers satisfied dependencies
[cylc]
    UTC mode = True
    [[events]]
        abort on stalled = True
[scheduling]
    initial cycle point = 2016
    final cycle point = 2016
    [[graph]]
        P1Y = """
holder => t1
holder => releaser
t1[-P1Y] => t1
"""
[runtime]
    [[t1]]
        script = true
    [[holder]]
        script = """
cylc hold "${CYLC_SUITE_NAME}" 't1.2016'
cylc__job__poll_grep_suite_log -F '[t1.20160101T0000Z] -waiting => waiting (held)'
"""
    [[releaser]]
        script = wait; cylc release "${CYLC_SUITE_NAME}" 't1.2016'
