[scheduling]
    initial cycle point = 20100101T00
    final cycle point = 20100102T00
    [[graph]]
        P1D = "daily"
        P1M = "monthly"
        P1Y = "yearly"
[runtime]
    [[root]]
        script = "true"
