[cylc]
   UTC mode = True
[scheduling]
    initial cycle point = 20130130T00
    final cycle point =   20130202T00
    runahead limit = PT12H # Enforces sequential behaviour
    [[graph]]
        # (this triggers a monthly task off the last daily task each month)
        T00 = """
            daily
            daily[-PT24H] => dummy
        """ # marker1
        R/01T/P1M = "dummy => monthly" # marker2
[runtime]
    [[root]]
        script = true
