[cylc]
    cycle point time zone = +01
[scheduling]
    initial cycle point = 20100101T00
    final cycle point   = +P5D
    [[graph]]
        T00 = "foo[-P1D] => foo"
[runtime]
    [[foo]]
        script = true
