[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20130808T00
    final cycle point = 20130808T18
    [[graph]]
        R1 = "setup_foo => foo"
        +PT6H/PT6H = "foo[-PT6H] => foo => bar"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 20130808T00
    final cycle point = 20130808T18
    [[node attributes]]
        foo = "color=red"
        bar = "color=orange"
