[scheduling]
    [[special tasks]]
        include at start-up = "include" # should be only task run
    [[graph]]
        R1 = "include & exclude"
[runtime]
    [[include]]
        script = true
    [[exclude]]
        script = false
