[scheduling]
    [[graph]]
        R1 = t1 & t2

[runtime]
    [[root]]
        script = true
    [[t1]]
    [[t2]]
        [[[job]]]
            submission polling intervals = 10*PT6S
            execution polling intervals = 10*PT6S
