[cylc]
    UTC mode = True
    cycle point format = %Y

[scheduling]
    initial cycle point = 2010
    final cycle point   = 2011
    [[graph]]
        P1Y = foo

[runtime]
    [[foo]]
        script = true
