[meta]
    title = cylc insert with wild card in task name string
[scheduling]
    [[special tasks]]
        exclude at start-up = t1, t2
    [[graph]]
        R1 = """
i1 => t1 & t2
"""
[runtime]
    [[i1]]
        script = """
cylc insert "${CYLC_SUITE_NAME}" 't?.1'
"""
    [[t1, t2]]
        script = true
