#!Jinja2

{% if not FOO is defined %}
    {{ raise('FOO must be defined.') }}
{% endif %}

{{ assert(ANSWER | int == 42, 'Universal constant incorrectly set.') }}

[scheduling]
    [[graph]]
        R1 = foo
