[cylc]
    UTC mode=True
[scheduling]
    initial cycle point=2020
    final cycle point=2020
    [[graph]]
        P1Y = t1
[runtime]
    [[t1]]
        script=test "${CYLC_TASK_SUBMIT_NUMBER}" -gt 2
        [[[job]]]
            execution retry delays=2*PT0S
