[scheduling]
   [[graph]]
       R1 = FAMILY
[runtime]
   [[FAMILY]]
   [[m1,m2,m3]]
      inherit = FAMILY
      [[[environment]]]
         FOO = foo
   [[m2]]
      [[[environment]]]
         FOO = bar
