[meta]
    title = "test suite for cylc-6 message triggers"

[cylc]
    UTC mode = True

[scheduling]
    initial cycle point = 20140801T00
    final cycle point = 20141201T00
    [[graph]]
        P2M = """
            foo:out1 => bar
            foo[-P2M]:out2 => baz
        """
[runtime]
    [[foo]]
        script = """
wait
cylc message -- "${CYLC_SUITE_NAME} "${CYLC_TASK_JOB} "file 1 done"
cylc message -- "${CYLC_SUITE_NAME} "${CYLC_TASK_JOB} "file 2 done"
"""
        [[[outputs]]]
            out1 = "file 1 done"
            out2 = "file 2 done"
    [[bar, baz]]
        script = true
