#!Jinja2
[cylc]
    UTC mode = True
    [[reference test]]
        expected task failures = local-1.1, local-2.1, local-3.1, remote-1.1, remote-2.1
[scheduling]
    [[graph]]
        R1="""
KILLABLE:start-all => killer
"""
[runtime]
    [[KILLABLE]]
        script=sleep 60
    [[local-1, local-2, local-3]]
        inherit = KILLABLE
    [[remote-1, remote-2]]
        inherit = KILLABLE
        [[[remote]]]
            host={{CYLC_TEST_HOST}}
    [[killer]]
        script="""
cylc kill "${CYLC_SUITE_NAME}" KILLABLE.1
cylc stop "${CYLC_SUITE_NAME}"
"""
