[cylc]
    UTC mode = True
    [[events]]
        timeout = PT0.2M
        abort on timeout = True
[scheduling]
    initial cycle point = 2010-01-01
    final cycle point = 2010-01-05
    runahead limit = PT12H # marker
    [[graph]]
        T00 = "foo:fail => reloader"
        T00/PT6H = "foo => bar"
[runtime]
    [[foo]]
        script = false
    [[bar]]
        script = true
    [[reloader]]
        script = """
# change the dependencies section name to garbage:
perl -pi -e 's/(runahead limit = )PT12H( # marker)/\1 PT18H\2/' $CYLC_SUITE_DEF_PATH/suite.rc
# reload
cylc reload -f $CYLC_SUITE_NAME
"""
