[meta]
    title = "Test insertion of a task that has previously run"

[cylc]
    UTC mode = True

[scheduling]
    initial cycle point = 20140101T00
    final cycle point   = 20140104T00
    [[graph]]
        R1 = "foo_cold => foo"
        T00 = "foo[-P1D] => foo"
        R1/+P1D = "foo[-P1D] => reinsert_foo => foo"

[runtime]
    [[foo_cold, foo]]
        script = "true"
    [[reinsert_foo]]
        script = """
            cylc insert $CYLC_SUITE_NAME foo_cold.20140101T0000Z
        """
