[meta]
    title = "Test insertion of a task added by a reload."
# Don't run this suite in-place: it modifies itself.

[scheduling]
   [[graph]]
       R1 = reloader

[runtime]
   [[root]]
      script = true
   [[reloader]]
      script = """
sed -i "s/\(R1 = reloader\)\s*$/\1 => add_me/" $CYLC_SUITE_DEF_PATH/suite.rc
cylc reload $CYLC_SUITE_NAME
cylc insert $CYLC_SUITE_NAME add_me.1
sleep 10
        """
