[cylc]
    cycle point num expanded year digits = 7
    UTC mode = True
[scheduling]
    initial cycle point = -13800000000-02-29T05:30  # Big Bang
    final cycle point = +05400000000-12-31T23:59    # Sun leaves main sequence
    [[graph]]
        R1 = "big_bang => count_the_aeons"
        +P1000000000Y/P1000000000Y = "count_the_aeons"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = -13800000000-02-29T05:30
    final cycle point = +05400000000-12-31T23:59
