#!Jinja2
[cylc]
    UTC mode = True
    [[events]]
        abort on timeout = True
        timeout = PT0.5M
[scheduling]
    initial cycle point = 20130101T00
    final cycle point   = 20150101T00
    cycling mode = 360day
    [[special tasks]]
        clock-trigger = clock(0)
    [[graph]]
        T00 = "clock"
[runtime]
    [[clock]]
        script = true
