[cylc]
    [[events]]
        abort on stalled = True
    [[reference test]]
        expected task failures = foo.1
[scheduling]
    [[graph]]
        R1 = """
            foo
            foo:fail => ! foo
        """
[runtime]
    [[foo]]
        script = """
            wait "${CYLC_TASK_MESSAGE_STARTED_PID}" 2>/dev/null || true
            trap '' EXIT
            trap '' ERR
            exit 1
        """
        [[[job]]]
            execution time limit = PT5S
