[scheduling]
    [[graph]]
        R1 = "foo:submit-fail => stopper"
[runtime]
    [[foo]]
        [[[job]]]
            batch system = at
            batch submit command template = sleep 30
    [[stopper]]
        script = cylc stop "${CYLC_SUITE_NAME}"
