#!jinja2
[cylc]
    [[events]]
        handlers = echo %(rubbish)s
        handler events = startup
{% if ABORT is defined %}
        abort if startup handler fails = True
{% endif %}

[scheduling]
    [[graph]]
        R1=t1

[runtime]
    [[t1]]
        script=true
