#!Jinja2

{% set HOST = environ['CYLC_TEST_HOST'] %}
{% set OWNER = environ['CYLC_TEST_OWNER'] %}

# whether or not ssh messaging is used back from the remote host is
# determined by the [hosts] section in site/user config files.
[scheduling]
    [[graph]]
        R1 = "foo => bar"
[runtime]
    [[foo]]
        script = "true" # fast
        [[[remote]]]
            host = {{ HOST }}
            owner = {{ OWNER }}
    [[bar]]
        script = "true"
        [[[remote]]]
            host = {{ HOST }}
