[cylc]
    cycle point format = %Y
[scheduling]
    initial cycle point = 2010
    final cycle point = 2012
    [[graph]]
        P1Y = "foo:x => bar"
[runtime]
    [[foo]]
        script = cylc message "the quick brown fox"
        [[[outputs]]]
            x = "the quick brown fox"
    [[bar]]
        script = true
