[cylc]
    UTC mode = True
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT3M
[scheduling]
    [[graph]]
        R1 = t1 => t2
[runtime]
    [[t1]]
        script=true
        [[[outputs]]]
            hello = Hello World
            greet = Greet World
    [[t2]]
        script = """
cylc reset --output='!hello' --output='!Greet World' "${CYLC_SUITE_NAME}" 't1.1'
cylc__job__poll_grep_suite_log -F '[t1.1] -reset output to incomplete: hello'
cylc__job__poll_grep_suite_log -F '[t1.1] -reset output to incomplete: Greet World'
cylc show "${CYLC_SUITE_NAME}" 't1.1' >"${CYLC_SUITE_RUN_DIR}/cylc-show.out"
"""
        [[[job]]]
            execution time limit = PT30S
