[cylc]
    cycle point format = %Y
    [[reference test]]
        expected task failures = get_obs.2010
[scheduling]
    initial cycle point = 2010
    final cycle point = 2013
    max active cycle points = 4
    spawn to max active cycle points = True
    [[graph]]
        P1Y = get_obs => proc_obs
        R1/P0Y = proc_obs => cleanup
[runtime]
    [[get_obs]]
        script = """
test "${CYLC_TASK_CYCLE_POINT}" != "${CYLC_SUITE_INITIAL_CYCLE_POINT}"
"""
    [[cleanup]]
        script = cylc reset -s succeeded $CYLC_SUITE_NAME get_obs:failed
