[meta]
    title = "cylc hold after point suite.rc test"

    description = """Define a hold after point in the suite.rc"""

[cylc]
    UTC mode = True

[scheduling]
    initial cycle point  = 20100101T00Z
    final cycle point    = 20100110T00Z
    hold after point    = 20100102T00Z
    [[graph]]
        R1 = stopper
        T00 = foo[-P1D] => foo
[runtime]
    [[stopper]]
        script = """
cylc__job__poll_grep_suite_log -F 'holding (beyond suite hold point) 20100102T00Z'
cylc stop "${CYLC_SUITE_NAME}"
"""
    [[foo]]
        script = true
