[meta]
    title=broadcast empty
    description=Test broadcast of an empty string
[scheduling]
    [[graph]]
        R1="broadcast => t1"
[runtime]
    [[broadcast]]
        script="""
cylc broadcast -s '[environment]EMPTY=' -p '1' -n 't1' "${CYLC_SUITE_NAME}" \
    | tee 'broadcast.out'
diff -u - 'broadcast.out' <<__OUT__
Broadcast set:
+ [t1.${CYLC_TASK_CYCLE_POINT}] [environment]EMPTY=
__OUT__
"""
    [[t1]]
        script = """
printenv EMPTY | tee 'echo.out'
diff -u - 'echo.out' <<<''
"""
        [[[environment]]]
            EMPTY=full
