[meta]
    title = "test insertion with a cycle point not on the tasks sequence"
[scheduling]
    cycling mode = integer
    initial cycle point = 0
    final cycle point = 4
    [[graph]]
        R1 = start => bad & qux
        P5 = baz[-P5] => bad => ugly => good => baz
        P10 = qux[-P5] => qux
[runtime]
    [[start]]
        script = """
cylc remove $CYLC_SUITE_NAME baz.0 --no-spawn
cylc remove $CYLC_SUITE_NAME qux.0 --no-spawn
"""
    [[bad]]
        script = "cylc insert $CYLC_SUITE_NAME baz.2"
    [[ugly]]
        script = "cylc insert $CYLC_SUITE_NAME qux.3 --no-check"
    [[good]]
        script = "cylc insert $CYLC_SUITE_NAME baz.0"
    [[baz, qux]]
        script = "echo ta-da!"
