[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20140101
    final cycle point = 20140104
    [[special tasks]]
        start-up = cold_foo
    [[graph]]
        0 = """
            cold_foo
            foo_midnight[T-24] & cold_foo => foo_midnight
        """
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 20140101
    final cycle point = 2014010206
