[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20000229T2000
    final cycle point = 20000301
    [[graph]]
        PT1H = "foo[-PT1H] => foo => bar"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 20000229T2000
    final cycle point = 20000301
