[scheduling]
    [[graph]]
        R1 = foo
[runtime]
    [[foo]]
        script = """
cylc message -p CUSTOM "the quick brown fox
jumped over the lazy dog"
"""
        [[[events]]]
            custom handler = echo %(message)s
