[cylc]
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = P3M
[scheduling]
    [[graph]]
        R1 = t1
[runtime]
    [[t1]]
        script = """
if ((CYLC_TASK_TRY_NUMBER == 1)); then
    cylc stop "${CYLC_SUITE_NAME}"
    cylc kill "${CYLC_SUITE_NAME}" "${CYLC_TASK_ID}"
    sleep 120  # Should not matter because the job will be killed
fi
"""
        [[[job]]]
            execution retry delays = PT0S
