[cylc]
   UTC mode = True
[scheduling]
    initial cycle point = 20130101T00
    final cycle point = 20130102T00
    [[graph]]
        PT6H = "foo[-PT6H] => foo"
[runtime]
    [[foo]]
        script = "true"
