#!jinja2
[cylc]
    UTC mode=True
    cycle point format = %Y
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = P1M
[scheduling]
    initial cycle point = 2016
    final cycle point = 2020
    [[graph]]
        P1Y=t2[-P1Y] => t1 => t2
[runtime]
    [[t1]]
        script = true
    [[t2]]
        script = false
