[cylc]
    cycle point time zone = +01
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT3M
[scheduling]
    initial cycle point = 20100101T00
    final cycle point   = T06
    [[graph]]
        T00 = "foo[-PT6H] => foo"
        T06 = "foo[-PT6H] => foo"
[runtime]
    [[foo]]
        script = true
