[cylc]
   UTC mode = True
[scheduling]
    initial cycle point = 20100101T00
    final cycle point = 20100102T00
    [[graph]]
        PT6H = "a[-PT6H] & b[-PT6H] => a => b"
[runtime]
    [[a, b]]
        script = true
