#!jinja2
[meta]
    title=Task Event Job Log Retrieve 1

[scheduling]
    [[graph]]
        R1=t1

[runtime]
    [[t1]]
        script=rm -f "${CYLC_TASK_LOG_ROOT}.err"
        [[[remote]]]
            host = {{HOST}}
            retrieve job logs = True
