#!jinja2

[meta]
    title=Test validation of blank script in automatic suite polling task
[scheduling]
    [[graph]]
        R1=eat<food::eat>=>full
[runtime]
    [[root]]
        script=true
    [[eat]]
        script=
        [[[suite state polling]]]
            interval=PT2S
            max-polls=20
    [[full]]
