#!jinja2
[meta]
    title=Task Event Mail

[cylc]
    task event mail interval = PT15S
    [[events]]
        abort on timeout = True
        mail footer = see: http://localhost/stuff/%(owner)s/%(suite)s/
        timeout = PT20S
    [[reference test]]
        expected task failures = t1.1, t2.1, t3.1, t4.1, t5.1

[scheduling]
    [[graph]]
        R1=t1 & t2 & t3 & t4 & t5

[runtime]
    [[t1, t2, t3, t4, t5]]
        script=false
        [[[job]]]
            execution retry delays = 2*PT20S
        [[[events]]]
            mail events = failed, retry
            mail smtp = {{MAIL_SMTP}}
