#!jinja2
[meta]
    title = "release family test suite"

    description = """One task that holds then selectively releases a family in the
first cycle point of the suite.."""

[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20141009T00
    [[graph]]
        R1 = holdrelease => foo & unrelated
        P1D = foo => bar => stop
[runtime]
    [[holdrelease]]
        script = """
wait
cylc hold "${CYLC_SUITE_NAME}"
cylc__job__poll_grep_suite_log -F 'INFO - Command succeeded: hold_suite()'
# Release a family in the first cycle point.
cylc release "$CYLC_SUITE_NAME" '{{RELEASE_MATCH}}.20141009T0000Z'
"""
    [[STUFF]]
    [[UNRELATED]]
    [[foo,bar]]
        inherit = STUFF
        script = true
    [[stop]]
        inherit = STUFF
        script = cylc stop "${CYLC_SUITE_NAME}"
    [[unrelated]]
        inherit = UNRELATED
        script = true
