[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 2015-02-21T00
    final cycle point = 2015-02-21T18
    [[graph]]
        PT6H = foo
        R1/P0D = foo => stop
        R20/PT6H/-PT6H = foo => nonstop
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 2015-02-21T00
    final cycle point = 2015-02-21T18
