[cylc]
    UTC mode = True # Ignore DST
    [[events]]
        stalled handler = cylc reset %(suite)s bar -s succeeded
    [[reference test]]
        expected task failures = bar.1
[scheduling]
    [[graph]]
        R1 = foo => bar => baz

[runtime]
    [[foo,baz]]
        script = true
    [[bar]]
        script = false
