[scheduling]
    [[graph]]
        R1 = t1 => t2 & t3
[runtime]
    [[t1]]
        script = """
ctb-get-graph-raw 'start_point_string=1' 'stop_point_string=1' 'group_nodes=T' \
    >"${CYLC_SUITE_RUN_DIR}/ctb-get-graph-raw.out"
"""
    [[T]]
        script = true
    [[t2, t3]]
        inherit = T
