[meta]
    title = "test suite for broadcast timeout functionality"

[cylc]
    UTC mode = True

[scheduling]
    initial cycle point = 20100808T0000Z
    final cycle point   = 20100808T0000Z
    [[graph]]
        R1 = send_broadcast => timeout

[runtime]
    [[send_broadcast]]
        script = """
            cylc broadcast -n timeout --point=20100808T0000Z --set='[events]execution timeout=PT1S' $CYLC_SUITE_NAME
        """
    [[timeout]]
        script = """
cylc__job__poll_grep_suite_log -F "[${CYLC_TASK_ID}] -execution timeout after PT1S"
"""
        [[[events]]]
            execution timeout = PT1M
