[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 00000101T00
    final cycle point = 00000101T00
    [[graph]]
        P1D = "foo[-P1Y] => foo"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 00000101T00
    final cycle point = 00000101T00
