[cylc]
    UTC mode = True # Ignore DST
    [[events]]
        abort on stalled = true
    [[reference test]]
        expected task failures = foo.1
[scheduling]
    [[graph]]
        R1 = """foo => FAM
            FAM:succeed-any => bar"""
[runtime]
    [[foo]]
        script = false
    [[FAM]]
    [[f_1, f_2, f_3]]
        inherit = FAM
