[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 1000
    final cycle point = 1001
    [[graph]]
        P1M = "foo[-P1M] => foo => bar"
        P6M = "baz[-P6M] => baz => qux"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 1000
    final cycle point = 1001
