[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20130228T00
    final cycle point = 20130301T00
    cycling mode = 360day
    [[graph]]
        P1D = foo[-P1D] => foo
[runtime]
    [[foo]]
        script = true
[visualization]
    initial cycle point = 20130228T00
    final cycle point = 20130301T00
