[scheduling]
    [[special tasks]]
        exclude at start-up = "exclude" # should not run
    [[graph]]
        R1 = "exclude & foo"
[runtime]
    [[foo]]
        script = true
    [[exclude]]
        script = false
