#!jinja2
[meta]
    title=Task Event Mail

[cylc]
{% if GLOBALCFG is not defined %}
    [[events]]
        mail footer = see: http://localhost/stuff/%(owner)s/%(suite)s/
{% endif %}{# not GLOBALCFG is not defined #}

[scheduling]
    [[graph]]
        R1=t1

[runtime]
    [[t1]]
        script=test "${CYLC_TASK_TRY_NUMBER}" -eq 2
        [[[job]]]
            execution retry delays = PT5S
{% if GLOBALCFG is not defined %}
        [[[events]]]
            mail events = failed, retry, succeeded
            mail smtp = {{MAIL_SMTP}}
{% endif %}{# not GLOBALCFG is not defined #}
