[meta]
    title = "hold/release test suite"
    description = """One task that holds then selectively releases tasks in the
    first cycle point of the suite. The spawned successors to those tasks should
    still be in the held state and should not run."""

[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20141009T00
    [[graph]]
        R1 = holdrelease => foo
        P1D = foo => bar => stop
[runtime]
    [[holdrelease]]
        script = """
wait
cylc hold "${CYLC_SUITE_NAME}"
cylc__job__poll_grep_suite_log -F 'INFO - Command succeeded: hold_suite()'
# Release all tasks in the first cycle point.
cylc release "${CYLC_SUITE_NAME}" '*.20141009T0000Z'
"""
    [[foo,bar]]
        script = true
    [[stop]]
        script = cylc stop "${CYLC_SUITE_NAME}"
