[meta]
    description = This suite is supposed to time out

[cylc]
    [[events]]
        timeout = PT6S
        abort on timeout = True

[scheduling]
    [[graph]]
        R1 = """foo
                   foo:fail => bar"""

[runtime]
    [[foo]]
        script = "true"
    [[bar]]
        # this task is never meant to run (thanks to the fail trigger),
        # it is just present to keep the suite alive after foo finishes
        # (preventing auto shutdown until the suite times out as intended).
