#!jinja2
[cylc]
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = P1M
[scheduling]
    [[graph]]
        R1 = t1
[runtime]
    [[t1]]
        script = true
