[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20150101
    final cycle point = 20200101
    [[graph]]
        P1Y = """
t1[-P2Y] => t1
t1 => t1-a => t1+a => t1%a
"""
[runtime]
    [[root]]
        script = printenv CYLC_TASK_ID
    [[t1, t1-a, t1+a, t1%a]]
