# Test automatic deprecation and deletion of config items as specified
# in cylc/flow/cfgspec/suite.py.

[cylc]
    [[dummy mode]]
    [[simulation mode]]
    [[events]]
        reset timer = 10
        reset inactivity timer = 15
[scheduling]
    initial cycle point = 20150808T00
    final cycle point = 20150808T00
    [[graph]]
        P1D = foo => cat & dog
    [[special tasks]]
        external-trigger = cat("meow available") # deprecated name
[runtime]
    [[foo, cat, dog]]
        [[[simulation mode]]]
            script = "echo script" # deprecate
        [[[dummy mode]]]
            script = "echo script" # deprecate
        [[[enable resurrection]]]
        [[[suite state polling]]]
            template = ""
        [[[events]]]
            reset timer = 20
[visualization]
    enable live graph movie = True # obsolete
