# Test job submission modules in lib/python/ and python/ (deprecated).
[scheduling]
    [[graph]]
        R1 = foo & bar

[runtime]
    [[root]]
        script = true # quick
    [[foo]]
        [[[job]]]
            # python/my_background.py  (deprecated location)
            batch system = my_background
    [[bar]]
        [[[job]]]
            # lib/python/my_background2.py
            batch system = my_background2
