[cylc]
   UTC mode = True
   [[reference test]]
       expected task failures = fixable.1
[scheduling]
    [[graph]]
        R1 = """
fixable:fail => fixer
fixable => z
"""
[runtime]
    [[fixable]]
        script = false
    [[fixer]]
        script = cylc reset --state='succeeded' "${CYLC_SUITE_NAME}" 'fixable' '1'
    [[z]]
        script = true
