[cylc]
   UTC mode = True
   [[reference test]]
       expected task failures = bar.20100809T0000Z

[scheduling]
    initial cycle point = 20100808T00
    final cycle point = 20100809T00
    [[graph]]
        T00 = """
            foo[-P1D]:restart1 | bar => foo
            foo => !bar
        """
[runtime]
    [[foo]]
        script = """
wait
cylc message "restart files ready"
"""
        [[[outputs]]]
            restart1 = "restart files ready"
    [[bar]]
        script = [[ "$(cylc cycle-point)" == 20100808T0000Z ]]
