#!Jinja2
[cylc]
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT3M
[scheduling]
    [[graph]]
        R1 = local-task:echo & remote-task:echo => cat-log
[runtime]
    [[ECHO]]
        script = """
wait
echo rubbish
echo garbage >&2
cylc message 'echo done'
"""
        [[[outputs]]]
            echo = "echo done"
    [[local-task]]
        inherit = ECHO
    [[remote-task]]
        inherit = ECHO
        [[[remote]]]
            host=${CYLC_TEST_HOST}
    [[cat-log]]
        script = """
wait
for TASK in local-task.1 remote-task.1; do
  cylc cat-log --debug -f o "${CYLC_SUITE_NAME}" $TASK | grep 'rubbish'
  cylc cat-log --debug -f e "${CYLC_SUITE_NAME}" $TASK | grep 'garbage'
done"""
