[cylc]
    UTC mode = True
[scheduling]
    [[graph]]
        R1 = "t1 => t2"
[runtime]
    [[t1]]
        script = """
cylc broadcast -n 't2' \
    -F "${CYLC_SUITE_DEF_PATH}/broadcast-1.rc" \
    -F "${CYLC_SUITE_DEF_PATH}/broadcast-2.rc" \
    "${CYLC_SUITE_NAME}"
"""
    [[t2]]
        script = false
