[meta]
    title = Test reload waiting task integrity error

[scheduling]
    [[graph]]
        R1 = """
sleeping-waiter => waiter
sleeping-waiter:start => reloader
"""

[runtime]
    [[sleeping-waiter]]
        script = """
touch 'file'
while [[ -e 'file' ]]; do
    sleep 1
done
"""
    [[waiter]]
        script = true
    [[reloader]]
        script = """
cylc reload "${CYLC_SUITE_NAME}"
cylc__job__poll_grep_suite_log '\[waiter\.1\] -reloaded task definition'
rm -f "${CYLC_SUITE_WORK_DIR}/1/sleeping-waiter/file"
"""
