[cylc]
    cycle point format = %Y
    [[parameters]]
        i = 1..500
[scheduling]
    initial cycle point = 2000
    final cycle point = 2010
    [[graph]]
        P1Y="""v<i>[-P1Y] => v<i>"""
[runtime]
    [[v<i>]]
        script=true
