#!Jinja2
[scheduling]
    [[graph]]
        R1 = t1

[runtime]
    [[t1]]
        script = sleep 60
{% if "CYLC_TEST_BATCH_TASK_HOST" in environ and environ["CYLC_TEST_BATCH_TASK_HOST"] %}
        [[[remote]]]
            host={{environ["CYLC_TEST_BATCH_TASK_HOST"]}}
{% endif %}
        [[[job]]]
            batch system = my_pbs
            execution time limit = PT2M
            execution polling intervals = PT20S
        [[[directives]]]
{% if "CYLC_TEST_BATCH_SITE_DIRECTIVES" in environ and
        environ["CYLC_TEST_BATCH_SITE_DIRECTIVES"] %}
    {{environ["CYLC_TEST_BATCH_SITE_DIRECTIVES"]}}
{% endif %}
