#!Jinja2
[cylc]
   [[events]]
       abort on timeout = True
       timeout = PT1M
[scheduling]
    [[graph]]
        R1 = a-task
[runtime]
    [[a-task]]
        script = echo "the quick brown fox"
        [[[remote]]]
            host={{environ['CYLC_TEST_HOST'] | default("localhost")}}
            owner={{environ['CYLC_TEST_OWNER'] | default(environ["USER"])}}
