#!Jinja2
[meta]
    title = "Jinja2 command line set test"
    description = """A couple of Jinja2 variables must be set on the command
line or else this test will fail to validate: TASKNAME and STEP. Also,
to pass the test run their values must be 'foo' and '2', respectively."""

[cylc]
    UTC mode = True

[scheduling]
    initial cycle point = 2010
    final cycle point = 2012
    [[graph]]
        P{{STEP}}Y = {{TASKNAME}}
[runtime]
    [[ {{TASKNAME}} ]]
        script = true
