[cylc]
    [[reference test]]
        expected task failures = t2.1
[scheduling]
    [[graph]]
        R1 = """
t1 => t2
t2:failed => t3
"""
[runtime]
    [[t1]]
        script = """
if ((${CYLC_TASK_SUBMIT_NUMBER} >= 2)); then
    cylc broadcast "${CYLC_SUITE_NAME}" -s '[environment]CYLC_TEST_VAR_X=x'
    cylc reset --state='waiting' "${CYLC_SUITE_NAME}" 't2.1'
fi
"""
    [[t2]]
        script = printenv CYLC_TEST_VAR_X
    [[t3]]
        script = cylc trigger "${CYLC_SUITE_NAME}" 't1.1'
