[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20000331T0100Z
    final cycle point = 2001
    [[graph]]
        20T-P2W/P1M = "foo"
        30T06+P8DT6H/P3W = "foo[20T-P2W] => bar"
        R1/+P1M = "flub"
        R4//P1D = "flub[^+P1M] & flob[-P1D] => flob"
[runtime]
    [[root]]
        script = true
