[meta]
    title = "A test with nested include-files"

[scheduling]
    initial cycle point = 20130101T00
    final cycle point = 20130101T00
    [[graph]]
        R1 = foo => bar
        T00 = bar
[runtime]
    [[root]]
        script = "echo Hello World"

[visualization]
    default node attributes = "style=filled"
