[scheduling]
    [[graph]]
        R1 = """FAM:finish-any => foo"""
[runtime]
    [[FAM]]
        script = sleep 10
    [[a,c]]
        inherit = FAM
    [[b]]
        inherit = FAM
        script = true
    [[foo]]
        script = true
