[cylc]
   UTC mode = True
[scheduling]
    initial cycle point = 1990
    final cycle point = 2030
    [[graph]]
        R1/1990 = """holder => releaser => t1"""
        R1/2010 = """releaser[1990] => t2"""
        R1/2030 = """
releaser[1990] => t3
releaser[1990] & t2[2010] & t3 => stopper
"""
[runtime]
    [[holder]]
        script = cylc hold "${CYLC_SUITE_NAME}" '*/t*'
    [[releaser]]
        script = cylc release "${CYLC_SUITE_NAME}" '20*/t*'
    [[stopper]]
        script = cylc stop "${CYLC_SUITE_NAME}"
    [[t1, t2, t3]]
        script = true
