#!Jinja2
[scheduling]
    [[graph]]
        R1 = a:start => b
[runtime]
    [[a]]
        script = sleep 20
{% if "CYLC_TEST_BATCH_TASK_HOST" in environ and environ["CYLC_TEST_BATCH_TASK_HOST"] %}
        [[[remote]]]
            host={{environ["CYLC_TEST_BATCH_TASK_HOST"]}}
{% endif %}
        [[[job]]]
            batch system = slurm
        [[[directives]]]
{% if "CYLC_TEST_BATCH_SITE_DIRECTIVES" in environ and
        environ["CYLC_TEST_BATCH_SITE_DIRECTIVES"] %}
    {{environ["CYLC_TEST_BATCH_SITE_DIRECTIVES"]}}
{% endif %}

    [[b]]
        script = cylc poll "$CYLC_SUITE_NAME" 'a'
