[cylc]
    [[events]]
        abort on stalled = True
[scheduling]
    [[graph]]
        R1 = foo
[runtime]
    [[foo]]
        init-script = """
CYLC_TEST_HOSTNAME="${HOSTNAME:-}"
"""
        script = test "${CYLC_TEST_HOSTNAME}" = "${HOSTNAME:-}"
