[cylc]
    UTC mode = True # Ignore DST
    [[events]]
        abort on stalled = true
    [[reference test]]
        expected task failures = bar.20100101T0000Z
[scheduling]
    initial cycle point = 20100101T0000Z
    [[graph]]
        T00, T06, T12, T18 = foo[-PT6H] & bar[-PT6H] => foo => bar => qux
        T12 = qux[-PT6H] => baz
[runtime]
    [[root]]
        script = true
    [[foo,baz, qux]]
    [[bar]]
        script = false
