[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 2000
    spawn to max active cycle points = True
    max active cycle points = 2
    [[graph]]
        T00 = a & b & (c | (d & e)) & f[-P1D] => f
[runtime]
    [[root]]
        script = true
    [[a, b, c, d, e, f]]
