#!jinja2
[meta]
    title = "hold tasks test suite"

    description = """One task that selectively holds tasks in the first cycle
point of the suite."""

[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20141009T00
    [[graph]]
        R1 = holdrelease => foo & stop
        P1D = foo
[runtime]
    [[holdrelease]]
        script = """
cylc hold "$CYLC_SUITE_NAME" '{{HOLD_MATCH}}.20141009T0000Z'
cylc__job__poll_grep_suite_log -F "[foo.20141009T0000Z] -waiting => waiting (held)"
"""
    [[foo]]
        script = true
    [[stop]]
        script = cylc stop "${CYLC_SUITE_NAME}"
