[cylc]
   [[reference test]]
       expected task failures = t1.1

[scheduling]
    [[graph]]
        R1 = """t1:start => t2"""

[runtime]
    [[t1]]
        script = sleep 60
    [[t2]]
        script = """
cylc shutdown "${CYLC_SUITE_NAME}"
sleep 1
cylc kill "${CYLC_SUITE_NAME}" 't1'
"""
