[cylc]
    [[events]]
        abort on stalled = True
[scheduling]
    initial cycle point = 1
    cycling mode = integer
    [[graph]]
        R1/+P1 = start
        R//P1 = start[2] => foo
[runtime]
    [[root]]
        script = true
