[meta]
    title = "hold/release a suite with tasks held beyond suite stop point"

    description = """
Releasing a held suite should not release tasks that are held because they're
beyond the suite stop point."""

# ref: GitHub Pull Request #1144

[cylc]
    cycle point format = %Y%m%dT%H

[scheduling]
    initial cycle point = 20140101T00
    final cycle point   = 20140101T00
    [[graph]]
        R1 = foo => holdrelease
        T00 = "foo => bar"
[runtime]
    [[holdrelease]]
        # When this task runs foo will be held beyond the suite stop point.
        script = """
cylc hold $CYLC_SUITE_NAME
cylc release $CYLC_SUITE_NAME"""
    [[foo]]
        script = true
    [[bar]]
        script = true
