#!Jinja2
[cylc]
   [[events]]
       abort on timeout = True
       timeout = PT2M
[scheduling]
    [[graph]]
        R1 = foo
[runtime]
    [[foo]]
        script = """
for I in $(seq 1 100); do
    echo "from $CYLC_TASK_NAME $I"
    sleep 1
done"""
        [[[remote]]]
            host={{environ['CYLC_TEST_HOST'] | default("localhost")}}
            owner={{environ['CYLC_TEST_OWNER'] | default(environ["USER"])}}
