[scheduling]
    [[graph]]
        R1 = bar
[runtime]
    [[bar]]
        script = """
wait "${CYLC_TASK_MESSAGE_STARTED_PID}" 2>/dev/null || true
sleep 2
cylc shutdown "${CYLC_SUITE_NAME}"
rm -f "${CYLC_SUITE_RUN_DIR}"
trap '' EXIT
exit
"""
