[scheduling]
    [[graph]]
            R1 = """
                    foo || bar => baz
            """
[runtime]
    [[foo]] 
        script = echo "foo"
        
    [[bar]]
        script = echo "bar"

    [[baz]]
        script = echo "baz"
