[cylc]
   UTC mode = True
[scheduling]
    initial cycle point = 2010-02
    final cycle point = 2010-08
    runahead limit = P1D # Enforce sequential running
    [[graph]]
        2010-01/P3M = "a"
[runtime]
    [[a]]
        script = true
