#!Jinja2
[cylc]
    UTC mode = {{UTC_MODE}}
    [[events]]
        abort on inactivity = True
        inactivity = {{TIMEOUT}}
[scheduling]
    initial cycle point = {{START}}
    final cycle point   = {{START}}
    [[special tasks]]
        clock-trigger = clock{% if OFFSET is defined %}({{OFFSET}}){% endif %}
    [[graph]]
        T{{HOUR}} = "clock"
[runtime]
    [[clock]]
        script = true
