#!jinja2
[meta]
    title = "Jinja2 simple ensemble example"
    description = "Auto-generation of dependencies for related tasks."

# Note that depending on the structure of the suite you may be able to
# SIMPLY use a task family name in the graph to represent the entire
# ensemble (which would be defined by inheritance under [runtime]).

%include inc/default.jinja2

# line \
continuation

[scheduling]
    [[graph]]
        R1 = """ {# generate ensemble dependencies #}
        {% for I in range( 0, N_MEMBERS ) %}
          foo => mem_{{ I }} => post_{{ I }} => bar
        {% endfor %}"""
