[scheduling]
    [[graph]]
        R1="w1:started => w2"

[runtime]
    [[w1]]
        script="""
# Turn off trap
wait "${CYLC_TASK_MESSAGE_STARTED_PID}"
trap '' EXIT
# Append to job.status
cat >>"${CYLC_TASK_LOG_ROOT}.status" <<__STATUS__
CYLC_JOB_EXIT=SUCCEEDED
CYLC_JOB_EXIT_TIME=$(date -u +'%FT%H:%M:%SZ')
__STATUS__
# Exit without trap
exit
"""
    [[w2]]
        script=wait; cylc poll "${CYLC_SUITE_NAME}" 'w1.1'
