[meta]
    title = "test insertion of a task with a bad cycle point."

[cylc]
    cycle point time zone = +01
    [[events]]
        abort on stalled = True

[scheduling]
    initial cycle point = 20140101T00
    final cycle point   = 20140101T00 # 3 cycles
    [[special tasks]]
        exclude at start-up = foo
    [[graph]]
        R1 = """
            prep
            foo => bar
        """

[runtime]
    [[root]]
        script = true
    [[prep]]
        # Insert the task with a bad cycle point
        script = cylc insert "$CYLC_SUITE_NAME" 'foo.teatime'
