[cylc]
    [[events]]
        abort on timeout = True
        timeout = PT20S
[scheduling]
    [[graph]]
        R1 = one => two
[runtime]
    [[one]]
        script = """
# Tell the suite to stop after I've finished.
cylc stop $CYLC_SUITE_NAME
sleep 10"""
    [[two]]
        script = true
