[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20131231T2300
    final cycle point = 20140101T0050
    [[graph]]
        PT30M = "foo[-PT30M] => foo => bar"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 20131231T2300
    final cycle point = 20140101T0050
