[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20130808T00
    final cycle point = 20130809T18
    [[graph]]
        R1 = "foo"
        R1/T12 = "foo[^] | bar => baz"
        T12 = "baz[-P1D] => baz"
[runtime]
    [[root]]
        script = true
    [[bar]]
        script = sleep 5
[visualization]
    initial cycle point = 20130808T00
    final cycle point = 20130809T18
    [[node attributes]]
        foo = "color=red"
        bar = "color=orange"
        baz = "color=green"
