#!jinja2
[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20100101T0000Z
    final cycle point   = 20110101T0000Z
    [[graph]]
        P1Y = poll_foo<{{REF_SUITE}}::foo>

[runtime]
    [[poll_foo]]
        [[[suite state polling]]]
            template = %Y
