[cylc]
   UTC mode = True
[scheduling]
    initial cycle point = 20100808T00
    final cycle point = 20100809T00
    [[graph]]
        T00 = "foo[-P1D]:restart1 => foo"
[runtime]
    [[foo]]
        script = """
wait
cylc message "restart files ready"
"""
        [[[outputs]]]
            restart1 = "restart files ready"
