#!jinja2
[cylc]
    UTC mode = False

[scheduling]
    [[graph]]
        R1=t1

[runtime]
    [[t1]]
        script=test -z "${TZ:-}"
        [[[job]]]
            execution time limit = PT1M
        [[[remote]]]
            host={{environ["CYLC_TEST_HOST"]}}
