#!jinja2
[meta]
    title=Suite Event Mail

[cylc]
    [[events]]
{% if GLOBALCFG is not defined %}
        mail events = startup, shutdown
        mail footer = see: http://localhost/stuff/%(owner)s/%(suite)s/
        mail smtp = {{MAIL_SMTP}}
{% endif %}{# not GLOBALCFG is not defined #}
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT3M

[scheduling]
    [[graph]]
        R1=t1

[runtime]
    [[t1]]
        script=true
