#!jinja2
{%- set TEST_DIR = environ['TEST_DIR'] %}
[cylc]
    UTC mode = True
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT3M
[scheduling]
    initial cycle point = 20130923T00
    final cycle point   = 20130923T00
    [[graph]]
        R1 = """
            failed_task:fail => shutdown
            shutdown => output_states
            output_states => finish => !failed_task
        """
[runtime]
    [[failed_task]]
        script = """
            exit 1
        """
        [[[meta]]]
            description = "Failed task (runs before restart)"
{% include 'suite-runtime-restart.rc' %}
