[cylc]
    UTC mode = True
    [[events]]
        abort on stalled = True
[scheduling]
    initial cycle point = 20000101T00Z
    [[graph]]
        R1 = start
        T00 = start[^] => foo
        R1/20000102T12Z = foo[20000102T00Z] => bar
[runtime]
    [[root]]
        script = true
