#!Jinja2
[cylc]
   [[events]]
       abort on stalled = True
   [[reference test]]
       expected task failures = foo.1
[scheduling]
    [[graph]]
        R1="foo:start => bar"

[runtime]
    [[bar]]
        script="""
wait "${CYLC_TASK_MESSAGE_STARTED_PID}" 2>/dev/null || true
cylc reload "${CYLC_SUITE_NAME}"
cylc__job__poll_grep_suite_log -F 'Reload completed'
cylc kill "${CYLC_SUITE_NAME}" 'foo.1'
cylc__job__poll_grep_suite_log -F '[foo.1] -job(01) killed'
"""
        [[[job]]]
            execution time limit = PT1M
    [[foo]]
        script=sleep 61
        [[[remote]]]
            host = {{CYLC_TEST_HOST}}
