[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20000101T00Z
    final cycle point = 20000102T12Z
    [[graph]]
        R1/^ = start => foo

        # Don't run at the initial cycle point
        # Also test whitespace tolerance
        T00 ! ^ = bar

        # Run 3-hourly but not 6 hourly.
        PT3H!PT6H = baz

        # Run 4 hourly but not at 0800 from ICP
        PT3H!T08 = qux

        # Run hourly but not at 00:00, 06:00, 12:00, 18:00
        T-00!(T00, T06, T12, T18) = nip

        # Run half-hourly but not on the hour from ICP
        PT30M!T-00 = quux

        # Run hourly on the hour, but not 6 hourly on the hour
        T-00!PT6H = wibble

        # Run hourly at 15 minutes past except every 3rd hour
        T-15!T-15/PT3H = bob
        # Run hourly at 15 minutes past except every 3rd hour
        # Implicit start point
        T-15!PT3H = toot
        # Stacked sequences
        T0230, PT3H!     (T09,    T12  ) , T1945 = dibble

        R1/$  = pub

[runtime]
    [[root]]
        script = echo success
[visualization]
    initial cycle point = 20000101T00Z
    final cycle point = 20000102T12Z
