[meta]
    title=broadcast section-space-key
    description=Test broadcast set section-space-key syntax
[cylc]
    UTC mode = True
    [[events]]
        abort if any task fails = True
        abort on timeout = True
        timeout=PT1M
[scheduling]
    initial cycle point = 20200202
    final cycle point = 20200202
    [[graph]]
        P1M = "broadcast => test-env"
[runtime]
    [[broadcast]]
        script="""
cylc broadcast -s '[environment] FOO=${FOO:-foo}' -n 'test-env' "${CYLC_SUITE_NAME}"
"""
    [[test-env]]
        script="""
test "${FOO}" = 'foo'
"""
        [[[environment]]]
            FOO=bar
