[cylc]
    [[events]]
        abort if any task fails = True
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT2M
[scheduling]
    [[graph]]
        R1 = bar => foo
[runtime]
    [[foo]]
        script = """
            cylc remove "${CYLC_SUITE_NAME}" 'bar.1'
            cylc insert "${CYLC_SUITE_NAME}" 'BAR.1'
        """
    [[BAR]]
    [[bar]]
        inherit = BAR
