#!jinja2
[cylc]
    UTC mode = True # Ignore DST

[scheduling]
    [[graph]]
        R1="t0 => t1"

[runtime]
    [[t0]]
        script = """
# A client command should not require --use-ssh option.
cylc broadcast "${CYLC_SUITE_NAME}" '--name=t1' '--set=script=true'
"""
        [[[remote]]]
            host = {{CYLC_TEST_HOST}}
    [[t1]]
        script = false
