[meta]
    title = "override this"
    title = "the quick brown fox"
[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20130101T0000Z
    [[graph]]
        R1 = "foo => bar"
        R1 = "bar => baz"
        T00 = "cfoo => cbar"
        T00 = "cbar => cbaz"
        T00 = "dfoo => dbar"
        T00 = "dbar => dbaz"
[runtime]
    [[FOO]]
        [[[meta]]]
            title = "override this"
            title = "override this too"
            description = "jumped over the lazy dog"
    [[FOO]]
        [[[meta]]]
            title = "the quick brown fox"
        [[[environment]]]
            VAR1 = "override this"
            VAR1 = "override this too"
            VAR1 = "the quick brown fox"
