[cylc]
   [[reference test]]
       expected task failures = foo.1

[scheduling]
    [[graph]]
        R1 = """foo:finish => bar
                   bar => !foo
                   baz:finish => qux"""
[runtime]
    [[foo]]
        script = false
    [[bar,baz,qux]]
        script = "true"
