[meta]
    title = A suite to test 'cylc trigger --edit'.
    description = """
A broken task will fail and cause the suite to abort on timeout, unless a
second task fixes and retriggers it with an edit-run."""
[cylc]
    [[events]]
        abort on timeout = True
        timeout = PT20S
[scheduling]
    [[graph]]
        R1 = """
            broken-task:fail => fixer-task
            syntax_errored_task:submit-fail => syntax_fixer_task
        """
[runtime]
    [[broken-task]]
        script = false
    [[fixer-task]]
        script = """
cylc trigger --edit $CYLC_SUITE_NAME 'broken-task.1' << __END1__
y
__END1__"""
    [[syntax_errored_task]]
        script = $(x
    [[syntax_fixer_task]]
        script = """
cylc trigger --edit $CYLC_SUITE_NAME 'syntax_errored_task.1' << __END2__
y
__END2__"""
