[cylc]
    UTC mode = true
[scheduling]
    initial cycle point = 20160101T00Z
    final cycle point = 20160102T00Z
    [[graph]]
        R1 = foo
        R1/^ = bar
        R1/^+PT1H = baz
        R1/$-PT1H = boo
        R1/$ = foo[^] & bar[^] & baz[^+PT1H] & boo[^+PT23H] => bot
[runtime]
    [[ root ]]
        script = echo "success"
