#!jinja2
[meta]
    title=Task Event Job Log Retrieve 1

[cylc]
    [[events]]
        abort on timeout = True
        timeout = PT1M
    [[reference test]]
        expected task failures = t1.1

[scheduling]
    [[graph]]
        R1=t1

[runtime]
    [[t1]]
        script=false
        err-script=rm -f "${CYLC_TASK_LOG_ROOT}.err"
        [[[remote]]]
            host = {{HOST}}
            retrieve job logs = True
            retrieve job logs retry delays = 2*PT5S
