[meta]
    title = A suite that should only succeed in dummy mode.
[cylc]
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT3M
[scheduling]
    [[graph]]
        R1 = foo:x => bar
[runtime]
    [[root]]
        [[[simulation]]]
            default run length = PT0S
            time limit buffer = PT1M
    [[bar]]
    [[foo]]
        script = false
        [[[outputs]]]
            x = "the quick brown fox"
