[scheduling]
    [[queues]]
        [[[my_queue]]]
            limit = 1
            members = METASYNTACTIC
    [[graph]]
        R1 = "foo:start => bar"
[runtime]
    [[METASYNTACTIC]]
    [[foo]]
        inherit = METASYNTACTIC
        script = """
            wait
            cylc trigger $CYLC_SUITE_NAME bar 1
        """
    [[bar]]
        inherit = METASYNTACTIC
        script = true
