[scheduling]
    initial cycle point = 20160101T00Z
    final cycle point = 20160102T00Z
    [[graph]]
        R1 = start
        PT1H = foo => bar => baz => bol
[runtime]
    [[root]]
        script = true
