[cylc]
    UTC mode = True
[scheduling]
    [[graph]]
        R1 = """
locker:lock => TRUES
"""
[runtime]
    [[locker]]
        script = timeout 400 cylc-db-lock
        [[[outputs]]]
            lock = "I have locked the public database file"
    [[TRUES]]
        script = true
    [[t0, t1, t2, t3, t4, t5, t6, t7, t8, t9]]
        inherit = TRUES
