[cylc]
    [[events]]
        abort if any task fails = True
[scheduling]
    [[graph]]
        R1 = foo
[runtime]
    [[foo]]
        script = true
        [[[environment]]]
            FOO=$(this is an error)
