[meta]
    title=broadcast bad point
    description=Test broadcast to an invalid cycle point fails.
# And see github #1415 - it did cause the suite server program to abort.
[cylc]
    [[events]]
        abort if any task fails = True
        abort on timeout = True
        timeout=PT1M
[scheduling]
    initial cycle point = 20150808
    final cycle point = 20150808
    [[graph]]
        P1M = broadcast
[runtime]
    [[broadcast]]
        script="""
# Broadcast to an integer point, not valid for this suite; and
# fail if the broadcast succeeds (it should fail).
! cylc broadcast -s 'title=foo' -p '1' "${CYLC_SUITE_NAME}"
"""
