[meta]
    title = cycling period change
    description = """change cycle points"""

[cylc]
   UTC mode = True

[scheduling]
    initial cycle point = 20100101T00
    final cycle point = 20100101T18
    [[graph]]
        T00 = reloader => a
        T00,T12 = a[-PT12H] => a
[runtime]
    [[reloader]]
        script = """
# change the order of FOO and BAR in the graphing section:
sed -i 's/T00,T12 = a\[-PT12H\]/T00,T06,T12,T18 = a[-PT6H]/' "${CYLC_SUITE_DEF_PATH}/suite.rc"
# reload
cylc reload -f "${CYLC_SUITE_NAME}"
cylc__job__poll_grep_suite_log -F 'Reload completed'
"""
    [[a]]
        script = true
