#!Jinja2
[cylc]
    UTC mode = True

[scheduling]
    initial cycle point=2020
    final cycle point=2025
    [[graph]]
        P1Y=T[-P1Y]:succeed-all => T
[runtime]
    [[T]]
        script = true
    [[t0,t1,t2,t3]]
        inherit = T
    [[t4,t5,t6]]
        inherit = T
        [[[remote]]]
            host = {{CYLC_TEST_HOST}}
