#!jinja2
{%- set TEST_DIR = environ['TEST_DIR'] %}
[cylc]
    UTC mode = True
    [[events]]
        abort on timeout = True
        timeout = PT3M
[scheduling]
    initial cycle point = 20130923T00
    final cycle point   = 20130923T00
    [[graph]]
        R1 = """
            submit_failed_task:submit-fail => shutdown
            shutdown => output_states
            output_states => finish => !submit_failed_task
        """
[runtime]
    [[submit_failed_task]]
        script = """
            exit 1  # Should not submit, so this shouldn't run!
        """
        [[[meta]]]
            description = "Submit-failed task (runs before restart)"
        [[[job]]]
            batch system = at
            batch submit command template = at oh-no
{% include 'suite-runtime-restart.rc' %}
