[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20131231T2300
    final cycle point = 20140103T0000
    [[graph]]
        P1D = "foo[-P1D] => foo => bar"
[runtime]
    [[root]]
        script = touch typing
[visualization]
    initial cycle point = 20131231T2300
    final cycle point = 20140103T0000
