#!Jinja2

[cylc]
    disable automatic shutdown = {{environ['SUITE_DISABLE_AUTO_SHUTDOWN'] | default(false)}}
    [[events]]
        timeout = PT2M
        abort on timeout = true
[scheduling]
    initial cycle point = 2020-01-01
    final cycle point = 2020-01-01
    [[graph]]
        P1D = foo
[runtime]
    [[foo]]
        script = true
