[cylc]
    cycle point time zone = Z
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT3M
[scheduling]
    runahead limit = PT18H
    initial cycle point = 20100101T00
    [[graph]]
        PT6H = "foo => bar"
        T12 = "foo:fail => shutdown"
[runtime]
    [[foo]]
        script = false
    [[bar]]
        script = true
    [[shutdown]]
        script = cylc shutdown "${CYLC_SUITE_NAME}"
