# A suite that will fail in live mode but pass in dummy and sim modes due to
# automatic completion of custom outputs.

[cylc]
    UTC mode = True
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT3M
[scheduling]
    initial cycle point = 2000
    final cycle point = 2000
    [[graph]]
        P1Y = "foo:meet & bar:greet => baz"
[runtime]
    [[root]]
        script = true
        [[[simulation]]]
            default run length = PT0S
    [[foo]]
        script = true
        [[[outputs]]]
            meet = meet
    [[bar]]
        script = true
        [[[outputs]]]
            greet = greet
    [[baz]]
