[cylc]
    UTC mode = True

[scheduling]
    initial cycle point = 20100101T0000Z
    final cycle point = 20100125T0000Z
    [[graph]]
        R5/W-1/P1W = foo

[runtime]
    [[foo]]
        script = true
