[cylc]
   [[events]]
       abort on stalled = True
       abort on inactivity = True
       inactivity = PT3M
[scheduling]
    [[graph]]
        R1 = foo
[runtime]
    [[foo]]
        script = test "${CYLC_TASK_TRY_NUMBER}" '-ge' '4'
        [[[job]]]
            execution retry delays = 3*PT0S
