[meta]
    title = """Job script: 
suite to environment test execution and submission polling intervals and task
communication method"""

[scheduling]
    [[graph]]
        R1 = "foo & bar"

[runtime]
    [[foo]]
    [[bar]]
        script = "sleep 10"
        [[[job]]]
            batch system = at
            batch submit command template = at now + 1 minutes
            submission polling intervals = PT0.1M
            execution polling intervals = PT0.3M, 2*PT0.2M, PT0.1M
