[cylc]
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT3M
[scheduling]
    [[queues]]
        [[[default]]]
            limit = 1
    [[graph]]
        R1 = m1 & m2
[runtime]
    [[m1, m2]]
        script = true
