[cylc]
    UTC mode = True
    [[events]]
        timeout = PT0.1M
        abort on timeout = True
[scheduling]
    initial cycle point = 20100101T00
    final cycle point = 20100105T00
    [[graph]]
        # T00, T07, T14, ...
        PT7H = "foo => bar"
        # T00, T12, T18... 
        T00, T12, T18 = "foo"
        # T04...
        T04 = "run_ok"
        # T05...
        T05 = "run_ok_2"
[runtime]
    [[foo]]
        script = false
    [[bar]]
        script = true
