#!Jinja2
[cylc]
    cycle point format = %Y-%m-%d
[scheduling]
    initial cycle point = 2015-01-01
    final cycle point   = 2015-01-03
    [[graph]]
      R1 = start => foo
      P1D = foo[-P1D] => foo => bar & baz
      R1/P0D = baz => stop
[visualization]
    {{STOP_CRITERION | default('')}}
    [[node attributes]]
        start = "style=filled", "fillcolor=slategray"
        stop = "style=filled", "fillcolor=red"
