#!jinja2
[cylc]
    disable automatic shutdown = True
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT1M
[scheduling]
    [[graph]]
        R1=t1
[runtime]
    [[t1]]
        script = true
