[cylc]
    UTC mode = true
    [[events]]
        abort on timeout = true
        timeout = PT20S
[scheduling]
    initial cycle point = 2015-01
    final cycle point = 2015-01
    [[graph]]
        P1D = foo
[runtime]
    [[foo]]
        script = true
