#!jinja2
[scheduling]
    [[graph]]
        R1=t1=>t2

[runtime]
    [[t1]]
        script="""
TIMEOUT=$(($(date +%s) + 120))
while [[ ! -e file ]] && (($TIMEOUT > $(date +%s))); do
    sleep 1
done
"""
        [[[job]]]
            batch system=background_vacation
    [[t2]]
        script=true
