[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20100101T00
    final cycle point = 20100105T00
    [[graph]]
        T00, T06, T12, T18 = "a[-PT6H] & c[-PT6H] => a => stopper => c"
[runtime]
    [[a,c]]
        script = "true"
    [[stopper]]
        script = "cylc shutdown $CYLC_SUITE_NAME 2010-01-01; sleep 5"
