[cylc]
   cycle point format = %Y
[scheduling]
    initial cycle point = 2020
    final cycle point = 2021
    [[graph]]
        P1Y = t1[-P1Y] => t1
[runtime]
    [[t1]]
        script = """
if [[ "${CYLC_TASK_CYCLE_POINT}" == '2020' ]]; then
    ctb-get-graph-raw \
        'start_point_string=2020' 'stop_point_string=None' 'group_nodes=T' \
        >"${CYLC_SUITE_RUN_DIR}/ctb-get-graph-raw.out"
fi
"""
