[cylc]
   [[events]]
      abort on timeout = True
      timeout = PT20S
[scheduling]
    [[graph]]
        R1 = "foo => stopper"
[runtime]
    [[foo]]
        [[[events]]]
            started handler = sleeper.sh %(id)s
    [[stopper]]
        script = cylc stop "${CYLC_SUITE_NAME}"
