#!jinja2
[meta]
    title=broadcast
    description=Test Broadcast Inheritance
[cylc]
    cycle point format = %Y%m%dT%H
[scheduling]
    initial cycle point = 20140101T00
    final cycle point   = 20140101T00
    [[graph]]
        R1 = broadcast_all => broadcast_tag
        T00 = broadcast_tag => t1
[runtime]
    [[broadcast_all]]
        script="""
cylc broadcast -s "[environment]ALL_0=true" -n F1 $CYLC_SUITE_NAME
cylc broadcast -s "[environment]ALL_1=true" -n t1 $CYLC_SUITE_NAME
"""
    [[broadcast_tag]]
        script="""
cylc broadcast -s "[environment]TAG_0=true" -n F1 -p $CYLC_TASK_CYCLE_POINT \
    $CYLC_SUITE_NAME
cylc broadcast -s "[environment]TAG_1=true" -n t1 -p $CYLC_TASK_CYCLE_POINT \
    $CYLC_SUITE_NAME
"""
    [[F1]]
        script="""
$ALL_0
$ALL_1
$TAG_0
$TAG_1
"""
        [[[environment]]]
            ALL_0=false
            ALL_1=false
            TAG_0=false
            TAG_1=false
    [[t1]]
        inherit=F1
