[cylc]
    UTC mode = True
[scheduling]
    [[graph]]
        R1 = "t1 => t2"
[runtime]
    [[t1]]
        script = """
cylc broadcast -n 't2' -F - "${CYLC_SUITE_NAME}" \
    <"${CYLC_SUITE_DEF_PATH}/broadcast.rc"
"""
    [[t2]]
        script = false
