[cylc]
    cycle point format = %Y
[scheduling]
    initial cycle point = now
    final cycle point = +P1Y
    max active cycle points = 1
    [[special tasks]]
        clock-trigger = foo(P0Y)
    [[graph]]
        P1Y = """
            foo => bar
            bar[-P1Y] => bar
        """ # (or sequential bar)
[runtime]
    [[root]]
        script = true
