#!jinja2
[meta]
    title=Suite Event Mail
    URL = http://mysuites.com/${CYLC_SUITE_NAME}.html
    suite-priority = HIGH

[cylc]
    [[events]]
{% if GLOBALCFG is not defined %}
        handlers = echo 'Your %(suite)s suite has a %(event)s event and URL %(suite_url)s and suite-priority as %(suite-priority)s and suite-UUID as %(suite_uuid)s.'
        handler events = startup
{% endif %}{# not GLOBALCFG is not defined #}

[scheduling]
    [[graph]]
        R1=t1

[runtime]
    [[t1]]
        script=true
