[cylc]
    cycle point time zone = Z
    [[events]]
        timeout = PT30S
        abort on timeout = True
[scheduling]
    runahead limit = PT18H
    initial cycle point = 20100101T00
    final cycle point = 20100105T00
    [[graph]]
        PT6H = "foo => bar"
[runtime]
    [[foo]]
        script = false
    [[bar]]
        script = true
