[cylc]
   UTC mode = True
[scheduling]
    [[graph]]
        R1 = t01
[runtime]
    [[t01]]
        script = true
        [[[events]]]
            handlers = my-handler "%(suite)s"
            handler events = succeeded
            handler retry delays = PT0S, PT10S
