[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20140101
    [[graph]]
        R1 = "cold_foo"
        PT12H = "cold_foo[^] & foo[-PT12H] => foo"
        R1/+P4D = "stop => foo"
[runtime]
    [[root]]
        script = true
    [[stop]]
        script = cylc shutdown "${CYLC_SUITE_NAME}" '20140107'
[visualization]
    initial cycle point = 20140101
    final cycle point = 20140110
