[cylc]
   [[reference test]]
       expected task failures = t1.1

[scheduling]
    [[graph]]
        R1 = """t1:submit-fail => t2"""

[runtime]
    [[t1]]
        script = true
        [[[job]]]
            batch system = bad
    [[t2]]
        script = """
grep -q -F \
    '1/t1/01|[STDERR] [Errno 2] No such file or directory: '"'"'bad-bad-bad-submit'"'" \
    "${CYLC_SUITE_LOG_DIR}/log"
cylc shutdown "${CYLC_SUITE_NAME}"
"""
