#!jinja2
[cylc]
    [[events]]
        abort on timeout = True
        timeout = PT30S
[scheduling]
    [[graph]]
        R1 = foo
[runtime]
    [[foo]]
        script = """
echo 'Hello to stdout'
>&2 echo 'Hello to stderr'
cat >> ${CYLC_TASK_LOG_ROOT}.custom <<__END__
the quick brown fox
jumped over
the lazy dog
__END__
"""
        [[[remote]]]
            host = {{CYLC_TEST_HOST | default("localhost")}}
            owner = {{CYLC_TEST_OWNER | default(environ["USER"])}}
            retrieve job logs = True
