#!jinja2
[cylc]
   [[events]]
       abort if any task fails = True
       abort on inactivity = True
       abort on stalled = True
       inactivity = PT2M
   [[reference test]]
       expected task failures = foo.1

[scheduling]
    [[graph]]
        R1 = foo

[runtime]
    [[foo]]
        script = sleep 10
        [[[job]]]
            batch system = {{CYLC_TEST_BATCH_SYS}}
            execution time limit = PT5S
