#!jinja2
[cylc]
    UTC mode=True
    [[events]]
        abort on inactivity = True
        inactivity = P1M
    [[reference test]]
        expected task failures = t1.19700101T0000Z
[scheduling]
    initial cycle point=1970
    final cycle point=1970
    [[graph]]
        P1Y=t1
[runtime]
    [[t1]]
        script=printenv CYLC_TEST_VAR_FOO
