[cylc]
   [[reference test]]
       expected task failures = t1.1

[scheduling]
    [[graph]]
        R1 = """t1:submit-fail => t2"""

[runtime]
    [[t1]]
        script = true
        [[[remote]]]
            host = $(cat 'rubbish.txt')
    [[t2]]
        script = """
cylc broadcast "${CYLC_SUITE_NAME}" -n 't1' -p '1' -s '[remote]host=localhost'
cylc trigger "${CYLC_SUITE_NAME}" 't1.1'
"""
