[meta]
    title = "Test suite for task state change on poll result."
    description = """Task A fails silently - it will be stuck in 'running'
unless polled. Task B then polls A to find it has failed, allowing A to
suicide via a :fail trigger, and the suite to shut down successfully."""

[cylc]
   [[reference test]]
       expected task failures = a.1

[scheduling]
    [[graph]]
        R1 = """
    a:start => b
    a:fail => !a"""

[runtime]
    [[a]]
        script = """
trap '' EXIT # die silently
exit 0"""
    [[b]]
        script = cylc poll "$CYLC_SUITE_NAME" 'a'
