#!Jinja2
[cylc]
   [[reference test]]
       expected task failures = t1.1, t2.1
[scheduling]
    [[graph]]
        R1="""
t1:start=>stop
t2:start=>stop
"""
[runtime]
    [[T]]
        script=sleep 120 & echo $! >file; wait
{% if "CYLC_TEST_HOST" in environ and environ["CYLC_TEST_HOST"] %}
        [[[remote]]]
            host={{environ["CYLC_TEST_HOST"]}}
{% endif %}
    [[t1]]
        inherit=T
    [[t2]]
        inherit=T
    [[stop]]
        script="""
cylc kill "$CYLC_SUITE_NAME" 't1.1' 't2.1' || true
cylc stop $CYLC_SUITE_NAME
"""
