[cylc]
    UTC mode = True
    [[events]]
        abort on stalled = True
[scheduling]
    initial cycle point = 20100101T0000Z
    final cycle point = +P1D
[[graph]]
    R1 = init_cycle
    R1/20100101T0600Z = fixed_cycle
    T12 = fixed_cycle[20100101T0600] => foo
[runtime]
    [[foo, fixed_cycle, init_cycle]]
        script = true
