#!jinja2
[scheduling]
    [[graph]]
        R1 = foo

[runtime]
    [[foo]]
        script = """
if [[ "${CYLC_TASK_SUBMIT_NUMBER}" == '1' ]]; then
    sleep 300
fi
"""
        [[[remote]]]
            host={{CYLC_TEST_BATCH_TASK_HOST}}
        [[[job]]]
            batch system = {{CYLC_TEST_BATCH_SYS}}
            execution time limit = PT70S
            execution retry delays = PT0S
        [[[directives]]]
            {{CYLC_TEST_BATCH_SITE_DIRECTIVES}}

