[meta]
    title = "cylc run --hold-after"

    description = """Test running with a hold-after cycle point."""

[cylc]
    UTC mode = True

[scheduling]
    initial cycle point = 20140101T00
    final cycle point   = 20140104T00
    max active cycle points = 10    
    [[graph]]
        R1 = stopper
        T00, T12 = foo[-PT12H] => foo => bar
[runtime]
    [[stopper]]
        script = sleep 30; cylc stop $CYLC_SUITE_NAME
    [[foo,bar]]
        script = true
