[cylc]
    UTC mode = True
    [[events]]
        abort on inactivity = True
        abort on stalled = True
        inactivity = PT1M

[scheduling]
    initial cycle point = 20100101T0000Z
    final cycle point = 20100102T0000Z
    [[graph]]
        T00, T06, T12, T18 = foo[-PT6H] => foo
        T18 = foo => inserter

[runtime]
    [[root]]
        script = true
    [[foo]]
    [[inserter]]
        script = """
cylc insert "${CYLC_SUITE_NAME}" 'foo.20100101T1200Z' --stop-point='20100101T1200Z'
"""
