#!jinja2
[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20100101T00Z
    final cycle point = 20100103T00Z
    [[graph]]
        T00 = "foo[-P1D] => foo"
        R1/20100102T00Z = """
            foo => env_polling
            foo => offset_polling
            foo[+P1D] => offset_polling2
        """
[runtime]
    [[foo]]
        script = true
    [[env_polling]]
        script = cylc suite-state $CYLC_SUITE_NAME --task=foo --task-point -S succeeded
    [[offset_polling]]
        script = cylc suite-state $CYLC_SUITE_NAME --task=foo -p 20100101T0000Z --offset=P1D
    [[offset_polling2]]
        script = cylc suite-state $CYLC_SUITE_NAME --task=foo -p 20100101T0000Z --offset=-P1D
