#!Jinja2
[cylc]
   [[events]]
       abort on stalled = True
   [[reference test]]
       expected task failures = t1.1
[scheduling]
    [[graph]]
        R1="""
t1:submit-fail => stopper
reloader => stopper
"""

[runtime]
    [[t1]]
        script=true
        [[[job]]]
            batch system = at
            batch submit command template = sleep 5
            submission retry delays = 3*PT5S
    [[reloader]]
        script=cylc reload "${CYLC_SUITE_NAME}"
    [[stopper]]
        script=cylc stop "${CYLC_SUITE_NAME}"
