#!Jinja2
[cylc]
   UTC mode = True
    [[events]]
        startup handler = cylc release '%(suite)s'
[scheduling]
    initial cycle point = 2016
    final cycle point = {{FINAL_CYCLE_POINT}}
    [[graph]]
        P1Y = foo
[runtime]
    [[foo]]
        script = {{COMMAND}}
