[cylc]
   UTC mode = True
   [[reference test]]
       expected task failures = fixable1.1, fixable2.1, fixable3.1
[scheduling]
    [[graph]]
        R1 = """
FIXABLES:fail-all => fixer
FIXABLES:succeed-all => Z
"""
[runtime]
    [[FIXABLES]]
        script = test "${CYLC_TASK_SUBMIT_NUMBER}" -eq 2
    [[fixable1, fixable2, fixable3]]
        inherit = FIXABLES
    [[fixer]]
        script = cylc trigger "${CYLC_SUITE_NAME}" '*:failed'
    [[Z]]
        script = true
    [[z1, z2]]
        inherit = Z
