#!Jinja2
[cylc]
{% if GLOBALCFG is not defined %}
    [[main loop]]
        [[[health check]]]
            interval = PT10S
{% endif %}{# not GLOBALCFG is not defined #}
    [[events]]
        abort if any task fails = True
        abort on stalled = False
        abort on timeout = True
        timeout = PT1M

[scheduling]
    [[graph]]
        R1 = t1

[runtime]
    [[t1]]
        script = """
wait "${CYLC_TASK_MESSAGE_STARTED_PID}" 2>/dev/null || true
# Remove contact file and don't report back to suite
SRVD="$(cylc get-global-config --print-run-dir)/${CYLC_SUITE_NAME}/.service"
rm -f "${SRVD}/contact"
trap '' EXIT
exit
"""
