[cylc]
   UTC mode = True
   [[reference test]]
       expected task failures = fixable-1a.1, fixable-1b.1, fixable-2a.1, fixable-2b.1, fixable-3.1, loser.1
[scheduling]
    [[graph]]
        R1 = """
FIXABLES:fail-all & loser:fail => fixer
FIXABLES:succeed-all & loser:fail => Z
Z:succeed-all => resetter
"""
[runtime]
    [[FIXABLES]]
        script = test "${CYLC_TASK_SUBMIT_NUMBER}" -eq 2
    [[FIXABLE-1, FIXABLE-2, FIXABLE-3]]
        inherit = FIXABLES
    [[fixable-1a, fixable-1b]]
        inherit = FIXABLE-1
    [[fixable-2a, fixable-2b]]
        inherit = FIXABLE-2
    [[fixable-3]]
        inherit = FIXABLE-3
    [[fixer]]
        script = """
cylc trigger "${CYLC_SUITE_NAME}" 'FIXABLE-1.*' 'fixable-2*.1' 'fixable-3.1'
"""
    [[loser]]
        script = false
    [[resetter]]
        script = cylc reset --state='succeeded' "${CYLC_SUITE_NAME}" 'loser.1'
    [[Z]]
        script = true
    [[z1, z2]]
        inherit = Z
