[cylc]
   UTC mode = True
   [[events]]
       abort on stalled = True
[scheduling]
    [[graph]]
        R1 = """
t1 => t3
t2 => t3
t1:hello & t2:greet => t4
"""
[runtime]
    [[t1]]
        script=true
        [[[outputs]]]
            hello = Hello World
    [[t2]]
        script=true
        [[[outputs]]]
            greet = Greet World
    [[t3]]
        script = """
cylc reset --debug --output=hello "${CYLC_SUITE_NAME}" 't1.1'
cylc__job__poll_grep_suite_log -F '[t1.1] -reset output to complete: hello'
cylc reset --debug --output='Greet World' "${CYLC_SUITE_NAME}" 't2.1'
cylc__job__poll_grep_suite_log -F '[t2.1] -reset output to complete: Greet World'
"""
        [[[job]]]
            execution time limit = PT30S
    [[t4]]
        script = true
