[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 1005
    final cycle point = 1014
    [[graph]]
        P1Y = "foo[-P1Y] => foo => bar"
        P4Y = "baz[-P4Y] => baz => qux"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 1005
    final cycle point = 1014
