[cylc]
    UTC mode = true
    [[events]]
        # Trigger the pre-start foo inserted by the inserter task.
        stalled handler = cylc trigger %(suite)s foo.20100101T1200Z
        abort on timeout = True
        timeout = PT2M

[scheduling]
    initial cycle point = 20100101T0000Z
    final cycle point   = 20100102T0000Z
    [[graph]]
        T00, T06, T12, T18 = bar[-PT6H] => foo => bar
        T18 = inserter => bar

[runtime]
    [[root]]
        script = true
    [[INSERT_FAM]]
    [[foo,bar]]
        inherit = INSERT_FAM
    [[inserter]]
        script = cylc insert $CYLC_SUITE_NAME INSERT_FAM.20100101T1200Z --stop-point=20100101T1200Z
