[meta]
    title = a suite with a task that multiply inherits from the same family
    description = should fail validation

[scheduling]
   [[graph]]
      R1 = foo

[runtime]
   [[A]]
   [[B]]
      inherit = C, A
   [[C]]
   [[foo]]
      inherit = A, B
