[cylc]
    UTC mode = True
    [[events]]
        abort on stalled = True
    [[reference test]]
        expected task failures = t1.1
[scheduling]
    [[graph]]
        R1 = """
            t0:failed & t1:failed => !t2
            t0 | t1 => t2
        """
[runtime]
    [[t0]]
        # https://github.com/cylc/cylc-flow/issues/2655
        # "t2.1" should not suicide on "t1.1:failed"
        script = cylc__job__poll_grep_suite_log '\[t1\.1\].*failed'
    [[t1]]
        script = false
    [[t2]]
        script = true
