[cylc]
   [[events]]
       abort on stalled = True
       abort on inactivity = True
       inactivity = PT3M
   [[reference test]]
       expected task failures = foo.1
[scheduling]
    [[graph]]
        R1 = "foo:submit-fail => !foo"
[runtime]
    [[foo]]
        script = true
        [[[job]]]
            submission retry delays = PT0S, PT0S, PT0S
        [[[remote]]]
            host = notahost
