[scheduling]
    [[graph]]
        R1 = """foo:a => bar
                   foo:b => baz"""
[runtime]
    [[root]]
        script = true
        [[[events]]]
            custom handler = echo !!CUSTOM!! %(name)s.%(point)s %(fish)s %(message)s
        [[[meta]]]
            fish = trout
    [[foo]]
        script = """
wait
# Output message for triggering, and custom event handler.
cylc message -p CUSTOM "Data ready for barring"
# Generic message, not for triggering or custom event handler.
cylc message "Aren't the hydrangeas nice?"
# Output message for triggering, not custom event handler.
cylc message "Data ready for bazzing"
"""
        [[[outputs]]]
            a = "Data ready for barring"
            b = "Data ready for bazzing"
        [[[meta]]]
            fish = fugu
