[cylc]
    UTC mode = True
    cycle point format = %Y

[scheduling]
    initial cycle point = 2020
    final cycle point = 2021
    [[graph]]
        R1 = remover
        # NOTE: Graph changed to make test pass on Python3.
        #       The reason for this is that without the dependency from
        #       remover[^] => foo there was no guarantee that remover would
        #       finish before baz ran.
        P1Y = remover[^] => foo => bar & baz
[runtime]
    [[remover]]
        script = """
# Remove first bar, after spawning it
# (it will run in the second cycle).
cylc remove $CYLC_SUITE_NAME bar.2020
# Remove first baz, without spawning it
# (so it will not run at all).
cylc remove --no-spawn $CYLC_SUITE_NAME baz.2020"""
    [[bar, baz]]
        script = true
