[cylc]
    cycle point format = %Y
[scheduling]
    initial cycle point = 2010
    final cycle point = 2011
    [[xtriggers]]
        x1 = faker(name="bob")
    [[graph]]
        R1 = "@x1 => foo => shutdown"
        P1Y = "@x1 => foo"
[runtime]
    [[foo]]
        script = "echo NAME is $x1_name"
    [[shutdown]]
        script = "cylc shutdown $CYLC_SUITE_NAME"
