[cylc]
    cycle point time zone = +13
[scheduling]
    initial cycle point = 20000131T0100Z
    final cycle point = 20000202T0600+0600
    [[graph]]
        PT3H = "foo[-PT3H] => foo => bar"
        PT6H = "baz[-PT6H] => baz => qux"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 20000131T0100Z
    final cycle point = 20000202T0600+0600
