#!jinja2   
    
{% set HELLO=False %}    
[  scheduling   ]    
    initial cycle point = 20140101T00
    [[  special tasks  ]]    
        sequential = foo , bar ,  baz   
        include at start-up =   \
            foo {% if HELLO %}   ,    \
            bar    
            {% endif %}   
    [[   graph  ]]   
        T00, T12  ="""   
        a => b   
        foo => bar &  baz  """   
        
   [  runtime   ]    
    [[  foo,bar   ,  a,  b  ]]   
%include "inc.rc"   
