[cylc]
    force run mode = dummy-local
[scheduling]
    [[graph]]
        R1 = oxygas
[runtime]
    [[root]]
        script = true
        [[[simulation]]]
            default run length = PT0S
    [[oxygas]]
        env-script = ELSE=foo
        [[[remote]]]
            host = whatever
        [[[environment]]]
            SOMETHING = "some-modification-$ELSE"
