#!empy
@{ MYVAR = "this is a set variable" }@

[scheduling]
    [[graph]]
        R1 = "a => FAM"
[runtime]
    [[a]]
        script = echo @(MYVAR)
    [[FAM]]
        [[[environment]]]
            TITLE="member"
@[ for num in range(5) ]@
    [[member_@num]]
        inherit = FAM
        script = echo I am $TITLE @num
@[ end for ]@
