[cylc]
   cycle point format = %Y
[scheduling]
   initial cycle point = 2010
   final cycle point = 2015
   [[graph]]
      P1Y = "foo:x => bar"
[runtime]
   [[root]]
      script = true
   [[bar]]
   [[foo]]
      script = cylc message "data ready"
      [[[outputs]]]
          x = "data ready"
