[meta]
    title = content reload test
    description = """two tasks: the second will fail, causing the test to
fail, unless the first reloads the suite definition after modifying it."""

[scheduling]
    [[graph]]
        R1 = "reloader => foo"

[runtime]
    [[reloader]]
        script = """
# change the value of $FALSE to "true" in foo's environment:
perl -pi -e 's/(FALSE = )false( # marker)/\1true\2/' $CYLC_SUITE_DEF_PATH/suite.rc
# reload
cylc reload -f $CYLC_SUITE_NAME
"""
    [[foo]]
        script = "$FALSE"
        [[[environment]]]
            FALSE = false # marker
