[cylc]
   [[reference test]]
       expected task failures = foo.1

[scheduling]
    [[graph]]
        R1 = """foo:fail => bar
                   bar => !foo"""
[runtime]
    [[foo]]
        script = "false"
    [[bar]]
        script = "true"
