[meta]
    title = "test insertion with and without a stop cycle on inserted tasks"

    description = """A cycling suite that runs for 3 cycles. Two of its
three tasks are excluded at start-up but are inserted manually by the 
initial prep task. One is inserted with no stop cycle, to run for the
duration of the suite; and the other is set to stop after two cycles."""

[cylc]
    cycle point format = %Y%m%dT%H

[scheduling]
    initial cycle point = 20140101T00
    final cycle point   = 20140103T00 # 3 cycles
    [[special tasks]]
        exclude at start-up = bar, baz
    [[graph]]
        R1 = prep => foo & bar & baz
        T00 = "foo & bar & baz"

[runtime]
    [[root]]
        script = true
    [[prep]]
        script = """
cylc insert $CYLC_SUITE_NAME bar.20140101T00
cylc insert --stop-point=20140102T00 $CYLC_SUITE_NAME baz.20140101T00
"""
