[cylc]
   UTC mode = True
[scheduling]
    initial cycle point = 20200202T0000Z
    final cycle point = 20200203T0000Z
    [[graph]]
        P1D = t1[-P1D] => t1
[runtime]
    [[t1]]
        script = """
if [[ "${CYLC_TASK_CYCLE_POINT}" == '20200202T0000Z' ]]; then
    ctb-get-graph-raw \
        'start_point_string=20200202T0000Z' \
        'stop_point_string=20200203T0000Z' \
        'group_nodes=T' \
        >"${CYLC_SUITE_RUN_DIR}/ctb-get-graph-raw.out"
fi
"""
