[cylc]
    [[events]]
        abort if any task fails = True

[scheduling]
    [[graph]]
        R1 = foo

[runtime]
    [[foo]]
        script = """
            # check PYTHONPATH is set correctly
            grep -q "${CYLC_SUITE_RUN_DIR}/lib/python" <<< "${PYTHONPATH}"

            # run a toy example
            python3 -c '
            from pub import beer
            assert beer.drink() == "98 bottles of beer on the wall."
            assert beer.drink() == "97 bottles of beer on the wall."
            assert beer.drink() == "96 bottles of beer on the wall."
            '
        """
