[cylc]
    cycle point time zone = +01
[scheduling]
    initial cycle point = 20100101T00
    final cycle point   = T06+PT06H
    [[graph]]
        T00, T06, T12, T18 = "foo[-P1D+PT18H] => foo"

[runtime]
    [[foo]]
        script = true
