#!jinja2
[cylc]
[scheduling]
    initial cycle point = 20140808T09
    final cycle point = 20140808T09
    [[special tasks]]
        clock-trigger = foo(PT5M)
    [[graph]]
        PT1H = """
            show => foo
            foo[-PT1H] => foo
            foo:start => baz & bar
        """
[runtime]
    [[foo]]
        script = true
    [[bar,baz]]
        script = true
    [[show]]
        script = """
cylc show "$CYLC_SUITE_NAME" foo.20140808T0900{{ TZ_OFFSET_BASIC }} >{{ TEST_SHOW_OUTPUT_PATH }}
"""
