[cylc]
    [[parameters]]
        i = 1..5
[scheduling]
    [[graph]]
        R1 = foo<i>
[runtime]
    [[foo<i>]]
        script = true
        [[[job]]]
            submission retry delays = PT10S
        [[[remote]]]
            host = $(my-host-select)
