#!jinja2

[meta]
    title = "polls for success and failure tasks in another suite"
[scheduling]
    [[graph]]
        R1 = """
          l-good<{{UPSTREAM}}::good-stuff> & lbad<{{UPSTREAM}}::bad:fail>
          l-mess<{{UPSTREAM}}::messenger> => done
                """
[runtime]
    [[l-good,lbad]]
        [[[suite state polling]]]
            interval = PT2S
            max-polls = 20
    [[l-mess]]
        [[[suite state polling]]]
            interval = PT2S
            max-polls = 20
            message = "the quick brown fox"
