[cylc]
    [[reference test]]
        expected task failures = foo.1
    [[events]]
        abort on stalled = True
[scheduling]
    [[graph]]
        R1 = foo:submit-failed => bar
[runtime]
    [[foo]]
        env-script = LETTERS=("a" "b" "c")
        pre-script = for letter in ${LETTERS[@]}
        script = do echo $letter
        post-script = done
    [[bar]]
        script = cylc stop "${CYLC_SUITE_NAME}"
