[scheduling]
    [[graph]] # marker
        R1 = reloader => foo
[runtime]
    [[reloader]]
        script = """
sleep 5
# change the dependencies section name to garbage:
perl -pi -e 's/(\[\[)graph(\]\] # marker)/\1garbage\2/' $CYLC_SUITE_DEF_PATH/suite.rc
# reload
cylc reload -f $CYLC_SUITE_NAME
"""
    [[foo]]
        script = true
