[cylc]
   UTC mode = True
   [[reference test]]
       expected task failures = fixable.19700101T0000Z, fixable.19900101T0000Z, fixable.20100101T0000Z
[scheduling]
    initial cycle point = 1970
    final cycle point = 2010
    [[graph]]
        P20Y = fixable
        R1/2010 = """
fixable[1970]:fail & fixable[1990]:fail => fixer
fixable:fail & fixer => resetter
fixable => z
"""
[runtime]
    [[fixable]]
        script = test "${CYLC_TASK_SUBMIT_NUMBER}" -eq 2
    [[fixer]]
        script = """
cylc trigger "${CYLC_SUITE_NAME}" '19*/*'
"""
    [[resetter]]
        script = """
cylc reset --state='succeeded' \
    "${CYLC_SUITE_NAME}" "*.${CYLC_TASK_CYCLE_POINT}:failed"
"""
    [[z]]
        script = true
