[meta]
    title = a suite with circular inheritance
    description = should fail validation

[scheduling]
   [[graph]]
      R1 = foo

[runtime]
   [[A]]
      inherit = B
   [[B]]
      inherit = C
   [[C]]
      inherit = B
   [[foo]]
      inherit = A
