[cylc]
    [[reference test]]
        expected task failures = foo.1
    [[events]]
        abort on stalled = True
[scheduling]
    [[graph]]
        R1 = foo
[runtime]
    [[foo]]
        script = """
if ((${CYLC_TASK_SUBMIT_NUMBER} == 1)); then
    false
    true
fi
"""
        [[[job]]]
            execution retry delays = PT0S
