[meta]
    title = A suite to test 'cylc trigger --edit' with host select command
    description = """
A broken task will fail and cause the suite to abort on timeout, unless a
second task fixes and retriggers it with an edit-run."""
[cylc]
    [[events]]
        abort on timeout = True
        timeout = PT20S
[scheduling]
    [[graph]]
        R1 = broken-task:fail => fixer-task
[runtime]
    [[broken-task]]
        script = false
        [[[remote]]]
            host = $(echo localhost)
    [[fixer-task]]
        script = """
cylc trigger --edit $CYLC_SUITE_NAME 'broken-task.1' << __END__
y
__END__"""
