[cylc]
   UTC mode = True
[scheduling]
    initial cycle point = 2011
    final cycle point = 2018
    runahead limit = P1D
    [[graph]]
        2010/P3Y = "a"
[runtime]
    [[a]]
        script = true
