[cylc]
    [[events]]
        abort on stalled = True
        abort on inactivity = True
        inactivity = PT3M
[scheduling]
    cycling mode = integer
    initial cycle point = 1
    [[graph]]
        R1/1 = """
            foo:fail => bar
            foo & bar => restart
        """
        R1/2 = """
            restart[-P1] => foo
            foo:fail => bar
            foo & bar => pub
        """
[runtime]
    [[foo]]
        script = false
    [[bar]]
        script = """
cylc reset "${CYLC_SUITE_NAME}" "foo.${CYLC_TASK_CYCLE_POINT}" -s succeeded
"""
    [[restart]]
        script = """
cylc stop "${CYLC_SUITE_NAME}"
"""
    [[pub]]
        script = """
# Extract timestamp lines from logs
for file in $(find "${CYLC_SUITE_RUN_DIR}/log/suite/" -name '*.*'); do
    grep '.*-.*-.*' "${file}" | sort -u || true
done | sort | uniq -d > 'log-duplication'
"""
