[cylc]
    UTC mode = True

[scheduling]
    initial cycle point = 20100101T0000Z
    [[graph]]
        R1 = t1:out1 => t2

[runtime]
    [[t1]]
        script = cylc task message "hello"
        [[[outputs]]]
            out1 = "hello"
    [[t2]]
        script = true
